@@ -15,32 +15,21 @@ __all__ = ['registered','failures','succeeds']
 # helper imports
 import warnings; warnings.filterwarnings("ignore", category=DeprecationWarning)
 import sys
-PY3 = (hex(sys.hexversion) >= '0x30000f0')
-if PY3:
-    import queue as Queue
-    import dbm as anydbm
-else:
-    import Queue
-    import anydbm
-    import sets # deprecated/removed
-    import mutex # removed
-try:
-    from cStringIO import StringIO # has StringI and StringO types
-except ImportError: # only has StringIO type
-    if PY3:
-        from io import BytesIO as StringIO
-    else:
-        from StringIO import StringIO
+import queue as Queue
+import dbm as anydbm
+from io import BytesIO as StringIO
 import re
 import array
 import collections
 import codecs
 import struct
+import dataclasses
 import datetime
 import calendar
 import weakref
 import pprint
 import decimal
+import numbers
 import functools
 import itertools
 import operator
@@ -56,6 +45,7 @@ import hashlib
 import hmac
 import os
 import logging
+import logging.handlers
 import optparse
 #import __hello__
 import threading
@@ -64,8 +54,7 @@ import contextlib
 try:
     import bz2
     import sqlite3
-    if PY3: import dbm.ndbm as dbm
-    else: import dbm
+    import dbm.ndbm as dbm
     HAS_ALL = True
 except ImportError: # Ubuntu
     HAS_ALL = False
@@ -112,7 +101,7 @@ class _newclass2(object):
 def _function(x): yield x
 def _function2():
     try: raise
-    except:
+    except Exception:
         from sys import exc_info
         e, er, tb = exc_info()
         return er, tb
@@ -123,20 +112,12 @@ if HAS_CTYPES:
 _filedescrip, _tempfile = tempfile.mkstemp('r') # deleted in cleanup
 _tmpf = tempfile.TemporaryFile('w')
 
-# put the objects in order, if possible
-try:
-    from collections import OrderedDict as odict
-except ImportError:
-    try:
-        from ordereddict import OrderedDict as odict
-    except ImportError:
-        odict = dict
 # objects used by dill for type declaration
-registered = d = odict()
+registered = d = {}
 # objects dill fails to pickle
-failures = x = odict()
+failures = x = {}
 # all other type objects
-succeeds = a = odict()
+succeeds = a = {}
 
 # types module (part of CH 8)
 a['BooleanType'] = bool(1)
@@ -157,12 +138,8 @@ a['ObjectType'] = object()
 a['StringType'] = _str = str(1)
 a['TupleType'] = _tuple = ()
 a['TypeType'] = type
-if PY3:
-    a['LongType'] = _int
-    a['UnicodeType'] = _str
-else:
-    a['LongType'] = long(1)
-    a['UnicodeType'] = unicode(1)
+a['LongType'] = _int
+a['UnicodeType'] = _str
 # built-in constants (CH 4)
 a['CopyrightType'] = copyright
 # built-in types (CH 5)
@@ -181,17 +158,13 @@ a['DefaultDictType'] = collections.defaultdict(_function, _dict)
 a['TZInfoType'] = datetime.tzinfo()
 a['DateTimeType'] = datetime.datetime.today()
 a['CalendarType'] = calendar.Calendar()
-if not PY3:
-    a['SetsType'] = sets.Set()
-    a['ImmutableSetType'] = sets.ImmutableSet()
-    a['MutexType'] = mutex.mutex()
 # numeric and mathematical types (CH 9)
 a['DecimalType'] = decimal.Decimal(1)
 a['CountType'] = itertools.count(0)
 # data compression and archiving (CH 12)
 a['TarInfoType'] = tarfile.TarInfo()
 # generic operating system services (CH 15)
-a['LoggerType'] = logging.getLogger()
+a['LoggerType'] = _logger = logging.getLogger()
 a['FormatterType'] = logging.Formatter() # pickle ok
 a['FilterType'] = logging.Filter() # pickle ok
 a['LogRecordType'] = logging.makeLogRecord(_dict) # pickle ok
@@ -199,21 +172,23 @@ a['OptionParserType'] = _oparser = optparse.OptionParser() # pickle ok
 a['OptionGroupType'] = optparse.OptionGroup(_oparser,"foo") # pickle ok
 a['OptionType'] = optparse.Option('--foo') # pickle ok
 if HAS_CTYPES:
-    a['CCharType'] = _cchar = ctypes.c_char()
-    a['CWCharType'] = ctypes.c_wchar() # fail == 2.6
-    a['CByteType'] = ctypes.c_byte()
-    a['CUByteType'] = ctypes.c_ubyte()
-    a['CShortType'] = ctypes.c_short()
-    a['CUShortType'] = ctypes.c_ushort()
-    a['CIntType'] = ctypes.c_int()
-    a['CUIntType'] = ctypes.c_uint()
-    a['CLongType'] = ctypes.c_long()
-    a['CULongType'] = ctypes.c_ulong()
-    a['CLongLongType'] = ctypes.c_longlong()
-    a['CULongLongType'] = ctypes.c_ulonglong()
-    a['CFloatType'] = ctypes.c_float()
-    a['CDoubleType'] = ctypes.c_double()
-    a['CSizeTType'] = ctypes.c_size_t()
+    z = x if IS_PYPY else a
+    z['CCharType'] = _cchar = ctypes.c_char()
+    z['CWCharType'] = ctypes.c_wchar() # fail == 2.6
+    z['CByteType'] = ctypes.c_byte()
+    z['CUByteType'] = ctypes.c_ubyte()
+    z['CShortType'] = ctypes.c_short()
+    z['CUShortType'] = ctypes.c_ushort()
+    z['CIntType'] = ctypes.c_int()
+    z['CUIntType'] = ctypes.c_uint()
+    z['CLongType'] = ctypes.c_long()
+    z['CULongType'] = ctypes.c_ulong()
+    z['CLongLongType'] = ctypes.c_longlong()
+    z['CULongLongType'] = ctypes.c_ulonglong()
+    z['CFloatType'] = ctypes.c_float()
+    z['CDoubleType'] = ctypes.c_double()
+    z['CSizeTType'] = ctypes.c_size_t()
+    del z
     a['CLibraryLoaderType'] = ctypes.cdll
     a['StructureType'] = _Struct
     # if not IS_PYPY:
@@ -223,40 +198,37 @@ if HAS_CTYPES:
 #NOTE: ctypes.c_int._objects is memberdescriptor for object's __dict__
 #NOTE: base class of all ctypes data types is non-public _CData
 
-try: # python 2.6
-    import fractions
-    import number
-    import io
-    from io import StringIO as TextIO
-    # built-in functions (CH 2)
-    a['ByteArrayType'] = bytearray([1])
-    # numeric and mathematical types (CH 9)
-    a['FractionType'] = fractions.Fraction()
-    a['NumberType'] = numbers.Number()
-    # generic operating system services (CH 15)
-    a['IOBaseType'] = io.IOBase()
-    a['RawIOBaseType'] = io.RawIOBase()
-    a['TextIOBaseType'] = io.TextIOBase()
-    a['BufferedIOBaseType'] = io.BufferedIOBase()
-    a['UnicodeIOType'] = TextIO() # the new StringIO
-    a['LoggingAdapterType'] = logging.LoggingAdapter(_logger,_dict) # pickle ok
-    if HAS_CTYPES:
-        a['CBoolType'] = ctypes.c_bool(1)
-        a['CLongDoubleType'] = ctypes.c_longdouble()
-except ImportError:
-    pass
-try: # python 2.7
-    import argparse
-    # data types (CH 8)
-    a['OrderedDictType'] = collections.OrderedDict(_dict)
-    a['CounterType'] = collections.Counter(_dict)
-    if HAS_CTYPES:
-        a['CSSizeTType'] = ctypes.c_ssize_t()
-    # generic operating system services (CH 15)
-    a['NullHandlerType'] = logging.NullHandler() # pickle ok  # new 2.7
-    a['ArgParseFileType'] = argparse.FileType() # pickle ok
-except (AttributeError, ImportError):
-    pass
+import fractions
+import io
+from io import StringIO as TextIO
+# built-in functions (CH 2)
+a['ByteArrayType'] = bytearray([1])
+# numeric and mathematical types (CH 9)
+a['FractionType'] = fractions.Fraction()
+a['NumberType'] = numbers.Number()
+# generic operating system services (CH 15)
+a['IOBaseType'] = io.IOBase()
+a['RawIOBaseType'] = io.RawIOBase()
+a['TextIOBaseType'] = io.TextIOBase()
+a['BufferedIOBaseType'] = io.BufferedIOBase()
+a['UnicodeIOType'] = TextIO() # the new StringIO
+a['LoggerAdapterType'] = logging.LoggerAdapter(_logger,_dict) # pickle ok
+if HAS_CTYPES:
+    z = x if IS_PYPY else a
+    z['CBoolType'] = ctypes.c_bool(1)
+    z['CLongDoubleType'] = ctypes.c_longdouble()
+    del z
+import argparse
+# data types (CH 8)
+a['OrderedDictType'] = collections.OrderedDict(_dict)
+a['CounterType'] = collections.Counter(_dict)
+if HAS_CTYPES:
+    z = x if IS_PYPY else a
+    z['CSSizeTType'] = ctypes.c_ssize_t()
+    del z
+# generic operating system services (CH 15)
+a['NullHandlerType'] = logging.NullHandler() # pickle ok  # new 2.7
+a['ArgParseFileType'] = argparse.FileType() # pickle ok
 
 # -- pickle fails on all below here -----------------------------------------
 # types module (part of CH 8)
@@ -275,45 +247,36 @@ a['ModuleType'] = datetime
 a['NotImplementedType'] = NotImplemented
 a['SliceType'] = slice(1)
 a['UnboundMethodType'] = _class._method #XXX: works when not imported!
-a['TextWrapperType'] = open(os.devnull, 'r') # same as mode='w','w+','r+'
-a['BufferedRandomType'] = open(os.devnull, 'r+b') # same as mode='w+b'
-a['BufferedReaderType'] = open(os.devnull, 'rb') # (default: buffering=-1)
-a['BufferedWriterType'] = open(os.devnull, 'wb')
+d['TextWrapperType'] = open(os.devnull, 'r') # same as mode='w','w+','r+'
+d['BufferedRandomType'] = open(os.devnull, 'r+b') # same as mode='w+b'
+d['BufferedReaderType'] = open(os.devnull, 'rb') # (default: buffering=-1)
+d['BufferedWriterType'] = open(os.devnull, 'wb')
 try: # oddities: deprecated
     from _pyio import open as _open
-    a['PyTextWrapperType'] = _open(os.devnull, 'r', buffering=-1)
-    a['PyBufferedRandomType'] = _open(os.devnull, 'r+b', buffering=-1)
-    a['PyBufferedReaderType'] = _open(os.devnull, 'rb', buffering=-1)
-    a['PyBufferedWriterType'] = _open(os.devnull, 'wb', buffering=-1)
+    d['PyTextWrapperType'] = _open(os.devnull, 'r', buffering=-1)
+    d['PyBufferedRandomType'] = _open(os.devnull, 'r+b', buffering=-1)
+    d['PyBufferedReaderType'] = _open(os.devnull, 'rb', buffering=-1)
+    d['PyBufferedWriterType'] = _open(os.devnull, 'wb', buffering=-1)
 except ImportError:
     pass
 # other (concrete) object types
-if PY3:
-    d['CellType'] = (_lambda)(0).__closure__[0]
-    a['XRangeType'] = _xrange = range(1)
-else:
-    d['CellType'] = (_lambda)(0).func_closure[0]
-    a['XRangeType'] = _xrange = xrange(1)
-if not IS_PYPY:
-    d['MethodDescriptorType'] = type.__dict__['mro']
-    d['WrapperDescriptorType'] = type.__repr__
-    a['WrapperDescriptorType2'] = type.__dict__['__module__']
-    d['ClassMethodDescriptorType'] = type.__dict__['__prepare__' if PY3 else 'mro']
+z = d if sys.hexversion < 0x30800a2 else a
+z['CellType'] = (_lambda)(0).__closure__[0]
+del z
+a['XRangeType'] = _xrange = range(1)
+a['MethodDescriptorType'] = type.__dict__['mro']
+a['WrapperDescriptorType'] = type.__repr__
+#a['WrapperDescriptorType2'] = type.__dict__['__module__']#XXX: GetSetDescriptor
+a['ClassMethodDescriptorType'] = type.__dict__['__prepare__']
 # built-in functions (CH 2)
-if PY3 or IS_PYPY: 
-    _methodwrap = (1).__lt__
-else: 
-    _methodwrap = (1).__cmp__
-d['MethodWrapperType'] = _methodwrap
+_methodwrap = (1).__lt__
+a['MethodWrapperType'] = _methodwrap
 a['StaticMethodType'] = staticmethod(_method)
 a['ClassMethodType'] = classmethod(_method)
 a['PropertyType'] = property()
 d['SuperType'] = super(Exception, _exception)
 # string services (CH 7)
-if PY3: 
-    _in = _bytes
-else: 
-    _in = _str
+_in = _bytes
 a['InputType'] = _cstrI = StringIO(_in)
 a['OutputType'] = _cstrO = StringIO()
 # data types (CH 8)
@@ -328,52 +291,40 @@ a['DeadCallableProxyType'] = weakref.proxy(_class2())
 a['QueueType'] = Queue.Queue()
 # numeric and mathematical types (CH 9)
 d['PartialType'] = functools.partial(int,base=2)
-if PY3:
-    a['IzipType'] = zip('0','1')
-else:
-    a['IzipType'] = itertools.izip('0','1')
+a['IzipType'] = zip('0','1')
 a['ChainType'] = itertools.chain('0','1')
 d['ItemGetterType'] = operator.itemgetter(0)
 d['AttrGetterType'] = operator.attrgetter('__repr__')
 # file and directory access (CH 10)
-if PY3: _fileW = _cstrO
-else: _fileW = _tmpf
+_fileW = _cstrO
 # data persistence (CH 11)
 if HAS_ALL:
-    a['ConnectionType'] = _conn = sqlite3.connect(':memory:')
-    a['CursorType'] = _conn.cursor()
+    x['ConnectionType'] = _conn = sqlite3.connect(':memory:')
+    x['CursorType'] = _conn.cursor()
 a['ShelveType'] = shelve.Shelf({})
 # data compression and archiving (CH 12)
 if HAS_ALL:
-    if (hex(sys.hexversion) < '0x2070ef0') or PY3:
-        a['BZ2FileType'] = bz2.BZ2File(os.devnull) #FIXME: fail >= 3.3, 2.7.14
-    a['BZ2CompressorType'] = bz2.BZ2Compressor()
-    a['BZ2DecompressorType'] = bz2.BZ2Decompressor()
-#a['ZipFileType'] = _zip = zipfile.ZipFile(os.devnull,'w') #FIXME: fail >= 3.2
+    x['BZ2FileType'] = bz2.BZ2File(os.devnull)
+    x['BZ2CompressorType'] = bz2.BZ2Compressor()
+    x['BZ2DecompressorType'] = bz2.BZ2Decompressor()
+#x['ZipFileType'] = _zip = zipfile.ZipFile(os.devnull,'w')
 #_zip.write(_tempfile,'x') [causes annoying warning/error printed on import]
 #a['ZipInfoType'] = _zip.getinfo('x')
 a['TarFileType'] = tarfile.open(fileobj=_fileW,mode='w')
 # file formats (CH 13)
-a['DialectType'] = csv.get_dialect('excel')
+x['DialectType'] = csv.get_dialect('excel')
 a['PackerType'] = xdrlib.Packer()
 # optional operating system services (CH 16)
 a['LockType'] = threading.Lock()
 a['RLockType'] = threading.RLock()
 # generic operating system services (CH 15) # also closed/open and r/w/etc...
-a['NamedLoggerType'] = _logger = logging.getLogger(__name__) #FIXME: fail >= 3.2 and <= 2.6
+a['NamedLoggerType'] = _logger = logging.getLogger(__name__)
 #a['FrozenModuleType'] = __hello__ #FIXME: prints "Hello world..."
 # interprocess communication (CH 17)
-if PY3:
-    a['SocketType'] = _socket = socket.socket() #FIXME: fail >= 3.3
-    a['SocketPairType'] = socket.socketpair()[0] #FIXME: fail >= 3.3
-else:
-    a['SocketType'] = _socket = socket.socket()
-    a['SocketPairType'] = _socket._sock
+x['SocketType'] = _socket = socket.socket()
+x['SocketPairType'] = socket.socketpair()[0]
 # python runtime services (CH 27)
-if PY3:
-    a['GeneratorContextManagerType'] = contextlib.contextmanager(max)([1])
-else:
-    a['GeneratorContextManagerType'] = contextlib.GeneratorContextManager(max)
+a['GeneratorContextManagerType'] = contextlib.contextmanager(max)([1])
 
 try: # ipython
     __IPYTHON__ is True # is ipython
@@ -390,59 +341,84 @@ try: # numpy #FIXME: slow... 0.05 to 0.1 sec to import numpy
     a['NumpyInt32Type'] = _numpy_int32
 except ImportError:
     pass
-try: # python 2.6
-    # numeric and mathematical types (CH 9)
-    a['ProductType'] = itertools.product('0','1')
-    # generic operating system services (CH 15)
-    a['FileHandlerType'] = logging.FileHandler(os.devnull) #FIXME: fail >= 3.2 and <= 2.6
-    a['RotatingFileHandlerType'] = logging.handlers.RotatingFileHandler(os.devnull)
-    a['SocketHandlerType'] = logging.handlers.SocketHandler('localhost',514)
-    a['MemoryHandlerType'] = logging.handlers.MemoryHandler(1)
-except AttributeError:
-    pass
-try: # python 2.7
-    # data types (CH 8)
-    a['WeakSetType'] = weakref.WeakSet() # 2.7
-#   # generic operating system services (CH 15) [errors when dill is imported]
-#   a['ArgumentParserType'] = _parser = argparse.ArgumentParser('PROG')
-#   a['NamespaceType'] = _parser.parse_args() # pickle ok
-#   a['SubParsersActionType'] = _parser.add_subparsers()
-#   a['MutuallyExclusiveGroupType'] = _parser.add_mutually_exclusive_group()
-#   a['ArgumentGroupType'] = _parser.add_argument_group()
-except AttributeError:
-    pass
+# numeric and mathematical types (CH 9)
+a['ProductType'] = itertools.product('0','1')
+# generic operating system services (CH 15)
+a['FileHandlerType'] = logging.FileHandler(os.devnull)
+a['RotatingFileHandlerType'] = logging.handlers.RotatingFileHandler(os.devnull)
+a['SocketHandlerType'] = logging.handlers.SocketHandler('localhost',514)
+a['MemoryHandlerType'] = logging.handlers.MemoryHandler(1)
+# data types (CH 8)
+a['WeakSetType'] = weakref.WeakSet() # 2.7
+# generic operating system services (CH 15) [errors when dill is imported]
+#a['ArgumentParserType'] = _parser = argparse.ArgumentParser('PROG')
+#a['NamespaceType'] = _parser.parse_args() # pickle ok
+#a['SubParsersActionType'] = _parser.add_subparsers()
+#a['MutuallyExclusiveGroupType'] = _parser.add_mutually_exclusive_group()
+#a['ArgumentGroupType'] = _parser.add_argument_group()
 
 # -- dill fails in some versions below here ---------------------------------
 # types module (part of CH 8)
-a['FileType'] = open(os.devnull, 'rb', buffering=0) # same 'wb','wb+','rb+'
-# FIXME: FileType fails >= 3.1
+d['FileType'] = open(os.devnull, 'rb', buffering=0) # same 'wb','wb+','rb+'
 # built-in functions (CH 2)
-a['ListIteratorType'] = iter(_list) # empty vs non-empty FIXME: fail < 3.2
-a['TupleIteratorType']= iter(_tuple) # empty vs non-empty FIXME: fail < 3.2
-a['XRangeIteratorType'] = iter(_xrange) # empty vs non-empty FIXME: fail < 3.2
+# Iterators:
+a['ListIteratorType'] = iter(_list) # empty vs non-empty
+a['SetIteratorType'] = iter(_set) #XXX: empty vs non-empty #FIXME: list_iterator
+a['TupleIteratorType']= iter(_tuple) # empty vs non-empty
+a['XRangeIteratorType'] = iter(_xrange) # empty vs non-empty
+a["BytesIteratorType"] = iter(b'')
+a["BytearrayIteratorType"] = iter(bytearray(b''))
+z = x if IS_PYPY else a
+z["CallableIteratorType"] = iter(iter, None)
+del z
+x["MemoryIteratorType"] = iter(memoryview(b''))
+a["ListReverseiteratorType"] = reversed([])
+X = a['OrderedDictType']
+d["OdictKeysType"] = X.keys()
+d["OdictValuesType"] = X.values()
+d["OdictItemsType"] = X.items()
+a["OdictIteratorType"] = iter(X.keys()) #FIXME: list_iterator
+del X
+#FIXME: list_iterator
+a['DictionaryItemIteratorType'] = iter(type.__dict__.items())
+a['DictionaryKeyIteratorType'] = iter(type.__dict__.keys())
+a['DictionaryValueIteratorType'] = iter(type.__dict__.values())
+if sys.hexversion >= 0x30800a0:
+    a["DictReversekeyiteratorType"] = reversed({}.keys())
+    a["DictReversevalueiteratorType"] = reversed({}.values())
+    a["DictReverseitemiteratorType"] = reversed({}.items())
+
+try:
+    import symtable
+    #FIXME: fails to pickle
+    x["SymtableEntryType"] = symtable.symtable("", "string", "exec")._table
+except ImportError:
+    pass
+
+if sys.hexversion >= 0x30a00a0:
+    x['LineIteratorType'] = compile('3', '', 'eval').co_lines()
+
+if sys.hexversion >= 0x30b00b0:
+    from types import GenericAlias
+    d["GenericAliasIteratorType"] = iter(GenericAlias(list, (int,)))
+    x['PositionsIteratorType'] = compile('3', '', 'eval').co_positions()
+
 # data types (CH 8)
-a['PrettyPrinterType'] = pprint.PrettyPrinter() #FIXME: fail >= 3.2 and == 2.5
+a['PrettyPrinterType'] = pprint.PrettyPrinter()
 # numeric and mathematical types (CH 9)
-a['CycleType'] = itertools.cycle('0') #FIXME: fail < 3.2
+a['CycleType'] = itertools.cycle('0')
 # file and directory access (CH 10)
-a['TemporaryFileType'] = _tmpf #FIXME: fail >= 3.2 and == 2.5
+a['TemporaryFileType'] = _tmpf
 # data compression and archiving (CH 12)
-a['GzipFileType'] = gzip.GzipFile(fileobj=_fileW) #FIXME: fail > 3.2 and <= 2.6
+x['GzipFileType'] = gzip.GzipFile(fileobj=_fileW)
 # generic operating system services (CH 15)
-a['StreamHandlerType'] = logging.StreamHandler() #FIXME: fail >= 3.2 and == 2.5
-try: # python 2.6
-    # numeric and mathematical types (CH 9)
-    a['PermutationsType'] = itertools.permutations('0') #FIXME: fail < 3.2
-    a['CombinationsType'] = itertools.combinations('0',1) #FIXME: fail < 3.2
-except AttributeError:
-    pass
-try: # python 2.7
-    # numeric and mathematical types (CH 9)
-    a['RepeatType'] = itertools.repeat(0) #FIXME: fail < 3.2
-    a['CompressType'] = itertools.compress('0',[1]) #FIXME: fail < 3.2
-    #XXX: ...and etc
-except AttributeError:
-    pass
+a['StreamHandlerType'] = logging.StreamHandler()
+# numeric and mathematical types (CH 9)
+a['PermutationsType'] = itertools.permutations('0')
+a['CombinationsType'] = itertools.combinations('0',1)
+a['RepeatType'] = itertools.repeat(0)
+a['CompressType'] = itertools.compress('0',[1])
+#XXX: ...and etc
 
 # -- dill fails on all below here -------------------------------------------
 # types module (part of CH 8)
@@ -452,16 +428,7 @@ x['TracebackType'] = _function2()[1] #(see: inspect.getouterframes,getframeinfo)
 # other (concrete) object types
 # (also: Capsule / CObject ?)
 # built-in functions (CH 2)
-x['SetIteratorType'] = iter(_set) #XXX: empty vs non-empty
 # built-in types (CH 5)
-if PY3:
-    x['DictionaryItemIteratorType'] = iter(type.__dict__.items())
-    x['DictionaryKeyIteratorType'] = iter(type.__dict__.keys())
-    x['DictionaryValueIteratorType'] = iter(type.__dict__.values())
-else:
-    x['DictionaryItemIteratorType'] = type.__dict__.iteritems()
-    x['DictionaryKeyIteratorType'] = type.__dict__.iterkeys()
-    x['DictionaryValueIteratorType'] = type.__dict__.itervalues()
 # string services (CH 7)
 x['StructType'] = struct.Struct('c')
 x['CallableIteratorType'] = _srepattern.finditer('')
@@ -471,7 +438,9 @@ x['StreamReader'] = codecs.StreamReader(_cstrI) #XXX: ... and etc
 # python object persistence (CH 11)
 # x['DbShelveType'] = shelve.open('foo','n')#,protocol=2) #XXX: delete foo
 if HAS_ALL:
-    x['DbmType'] = dbm.open(_tempfile,'n')
+    z = a if IS_PYPY else x
+    z['DbmType'] = dbm.open(_tempfile,'n')
+    del z
 # x['DbCursorType'] = _dbcursor = anydbm.open('foo','n') #XXX: delete foo
 # x['DbType'] = _dbcursor.db
 # data compression and archiving (CH 12)
@@ -484,7 +453,7 @@ x['CSVDictReaderType'] = csv.DictReader(_cstrI)
 x['CSVDictWriterType'] = csv.DictWriter(_cstrO,{})
 # cryptographic services (CH 14)
 x['HashType'] = hashlib.md5()
-if (hex(sys.hexversion) < '0x30800a1'):
+if (sys.hexversion < 0x30800a1):
     x['HMACType'] = hmac.new(_in)
 else:
     x['HMACType'] = hmac.new(_in, digestmod='md5')
@@ -512,41 +481,46 @@ if HAS_CTYPES:
     x['NullPtrType'] = _lpchar()
     x['NullPyObjectType'] = ctypes.py_object()
     x['PyObjectType'] = ctypes.py_object(lambda :None)
-    x['FieldType'] = _field = _Struct._field
-    x['CFUNCTYPEType'] = _cfunc = ctypes.CFUNCTYPE(ctypes.c_char)
+    z = a if IS_PYPY else x
+    z['FieldType'] = _field = _Struct._field
+    z['CFUNCTYPEType'] = _cfunc = ctypes.CFUNCTYPE(ctypes.c_char)
     x['CFunctionType'] = _cfunc(str)
-try: # python 2.6
-    # numeric and mathematical types (CH 9)
-    x['MethodCallerType'] = operator.methodcaller('mro') # 2.6
-except AttributeError:
-    pass
-try: # python 2.7
-    # built-in types (CH 5)
-    x['MemoryType'] = memoryview(_in) # 2.7
-    x['MemoryType2'] = memoryview(bytearray(_in)) # 2.7
-    if PY3:
-        x['DictItemsType'] = _dict.items() # 2.7
-        x['DictKeysType'] = _dict.keys() # 2.7
-        x['DictValuesType'] = _dict.values() # 2.7
-    else:
-        x['DictItemsType'] = _dict.viewitems() # 2.7
-        x['DictKeysType'] = _dict.viewkeys() # 2.7
-        x['DictValuesType'] = _dict.viewvalues() # 2.7
-    # generic operating system services (CH 15)
-    x['RawTextHelpFormatterType'] = argparse.RawTextHelpFormatter('PROG')
-    x['RawDescriptionHelpFormatterType'] = argparse.RawDescriptionHelpFormatter('PROG')
-    x['ArgDefaultsHelpFormatterType'] = argparse.ArgumentDefaultsHelpFormatter('PROG')
-except NameError:
-    pass
-try: # python 2.7 (and not 3.1)
-    x['CmpKeyType'] = _cmpkey = functools.cmp_to_key(_methodwrap) # 2.7, >=3.2
-    x['CmpKeyObjType'] = _cmpkey('0') #2.7, >=3.2
-except AttributeError:
-    pass
-if PY3: # oddities: removed, etc
-    x['BufferType'] = x['MemoryType']
-else:
-    x['BufferType'] = buffer('')
+    del z
+# numeric and mathematical types (CH 9)
+a['MethodCallerType'] = operator.methodcaller('mro') # 2.6
+# built-in types (CH 5)
+x['MemoryType'] = memoryview(_in) # 2.7
+x['MemoryType2'] = memoryview(bytearray(_in)) # 2.7
+d['DictItemsType'] = _dict.items() # 2.7
+d['DictKeysType'] = _dict.keys() # 2.7
+d['DictValuesType'] = _dict.values() # 2.7
+# generic operating system services (CH 15)
+a['RawTextHelpFormatterType'] = argparse.RawTextHelpFormatter('PROG')
+a['RawDescriptionHelpFormatterType'] = argparse.RawDescriptionHelpFormatter('PROG')
+a['ArgDefaultsHelpFormatterType'] = argparse.ArgumentDefaultsHelpFormatter('PROG')
+z = a if IS_PYPY else x
+z['CmpKeyType'] = _cmpkey = functools.cmp_to_key(_methodwrap) # 2.7, >=3.2
+z['CmpKeyObjType'] = _cmpkey('0') #2.7, >=3.2
+del z
+# oddities: removed, etc
+x['BufferType'] = x['MemoryType']
+
+from dill._dill import _testcapsule
+if _testcapsule is not None:
+    d['PyCapsuleType'] = _testcapsule
+del _testcapsule
+
+if hasattr(dataclasses, '_HAS_DEFAULT_FACTORY'):
+    a['DataclassesHasDefaultFactoryType'] = dataclasses._HAS_DEFAULT_FACTORY
+
+if hasattr(dataclasses, 'MISSING'):
+    a['DataclassesMissingType'] = dataclasses.MISSING
+
+if hasattr(dataclasses, 'KW_ONLY'):
+    a['DataclassesKWOnlyType'] = dataclasses.KW_ONLY
+
+if hasattr(dataclasses, '_FIELD_BASE'):
+    a['DataclassesFieldBaseType'] = dataclasses._FIELD
 
 # -- cleanup ----------------------------------------------------------------
 a.update(d) # registered also succeed
